/**
 * @author Vladimir Kozhin <affka@affka.ru>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _Component2 = require('../base/Component');

var _Component3 = _interopRequireDefault(_Component2);

var _BrowserHelper = require('../helpers/BrowserHelper');

var _BrowserHelper2 = _interopRequireDefault(_BrowserHelper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DropArea = function (_Component) {
    _inherits(DropArea, _Component);

    function DropArea() {
        _classCallCheck(this, DropArea);

        return _possibleConstructorReturn(this, (DropArea.__proto__ || Object.getPrototypeOf(DropArea)).apply(this, arguments));
    }

    _createClass(DropArea, [{
        key: 'preInit',
        value: function preInit() {
            /**
             * @type {boolean}
             */
            this._enable = false;

            /**
             * @type {HTMLElement}
             */
            this.container = null;

            /**
             * @type {HTMLElement}
             */
            this._mask = null;

            /**
             * @type {object}
             */
            this._files = {};

            this._readLevel = 0;

            _get(DropArea.prototype.__proto__ || Object.getPrototypeOf(DropArea.prototype), 'preInit', this).apply(this, arguments);
        }
    }, {
        key: 'init',
        value: function init() {
            this.container = this.container || document.body;
            this._initContainerEvents();
            this._initMask();
        }
    }, {
        key: '_initContainerEvents',
        value: function _initContainerEvents() {
            if (this.container) {
                this.container.ondragover = this._enable ? this._onDragOver.bind(this) : null;
                this.container.ondragleave = this._enable ? this._onDragLeave.bind(this) : null;
                this.container.ondrop = this._enable ? this._onDrop.bind(this) : null;
            }
        }
    }, {
        key: '_initMask',
        value: function _initMask() {
            this._mask = document.createElement('div');
            this._mask.style.position = 'absolute';
            this._mask.style.top = 0;
            this._mask.style.left = 0;
            this._mask.style.right = 0;
            this._mask.style.bottom = 0;
            this._mask.style.background = 'transparent';
            this._mask.style['z-index'] = 10000;
            this._mask.style.display = 'none';

            this.container.appendChild(this._mask);

            this._mask.ondragover = this._enable ? this._onDragOver.bind(this) : null;
            this._mask.ondragleave = this._enable ? this._onDragLeave.bind(this) : null;
            this._mask.ondrop = this._enable ? this._onDrop.bind(this) : null;
        }

        /**
         *
         * @param {boolean} value
         */

    }, {
        key: 'setEnable',
        value: function setEnable(value) {
            this._enable = value && _BrowserHelper2.default.isFileDropSupport();
            this._initContainerEvents();
        }

        /**
         *
         * @returns {boolean}
         */

    }, {
        key: 'getEnable',
        value: function getEnable() {
            return this._enable;
        }
    }, {
        key: '_onDragOver',
        value: function _onDragOver(event) {
            event.preventDefault();

            if (event.target !== this._mask) {
                this._mask.style.display = 'block';
            }

            if (event.dataTransfer) {
                var dtTypes = event.dataTransfer.types;
                if (dtTypes) {
                    // FF
                    if (dtTypes.contains && !dtTypes.contains('Files')) {
                        return;
                    }

                    // Chrome
                    if (dtTypes.indexOf && dtTypes.indexOf('Files') === -1) {
                        return;
                    }
                }

                event.dataTransfer.dropEffect = 'copy';
            }

            this.trigger(DropArea.EVENT_DRAG_OVER, [event]);
        }
    }, {
        key: '_onDragLeave',
        value: function _onDragLeave(event) {
            event.preventDefault();

            if (event.target === this._mask) {
                this._mask.style.display = 'none';
            }

            this.trigger(DropArea.EVENT_DRAG_LEAVE, [event]);
        }
    }, {
        key: '_onDrop',
        value: function _onDrop(event) {
            event.preventDefault();

            this._mask.style.display = 'none';

            if (event.dataTransfer.items && event.dataTransfer.items.length > 0) {
                this._readDataTransferItems(event.dataTransfer.items);
            } else {
                this._readDataTransferFiles(event.dataTransfer.files);
            }

            this.trigger(DropArea.EVENT_DROP, [event]);
        }
    }, {
        key: '_readDataTransferItems',
        value: function _readDataTransferItems(items) {
            var entries = [];
            for (var i = 0, l = items.length; i < l; i++) {
                var entry = items[i].webkitGetAsEntry();
                if (entry !== null) {
                    entries.push(entry);
                }
            }

            this._readDirectoryEntries(entries, '');
        }
    }, {
        key: '_readDirectoryEntries',
        value: function _readDirectoryEntries(entries, relativePath) {
            var _this2 = this;

            this._readLevel++;

            for (var i = 0, l = entries.length; i < l; i++) {
                (function (entry) {
                    var path = (relativePath ? relativePath + '/' : '') + entry.name;

                    if (entry.isDirectory) {
                        entry.createReader().readEntries(function (subEntries) {
                            _this2._readDirectoryEntries(subEntries, path);
                        }, function (e) {
                            throw e;
                        });
                        _this2._readDirectoryEntries(entry, entry.name + '/');
                    } else if (entry.isFile) {
                        _this2._readLevel++;
                        entry.file(function (file) {
                            _this2._files[path] = file;

                            _this2._readLevel--;
                            _this2._readDirectoryEntries([], path);
                        }, function (e) {
                            throw e;
                        });
                    }
                })(entries[i]);
            }
            this._readLevel--;

            if (this._readLevel === 0) {
                this._onReadDataTransfer();
            }
        }
    }, {
        key: '_readDataTransferFiles',
        value: function _readDataTransferFiles(files) {
            for (var i = 0, l = files.length; i < l; i++) {
                var file = files[i];

                // Skip folders
                if (!file.type && file.size === 0) {
                    continue;
                }

                this._files[file.name] = file;
            }

            this._onReadDataTransfer();
        }
    }, {
        key: '_onReadDataTransfer',
        value: function _onReadDataTransfer() {
            this.trigger(DropArea.EVENT_SUBMIT, [this._files]);
            this._files = {};
        }
    }]);

    return DropArea;
}(_Component3.default);

exports.default = DropArea;


DropArea.EVENT_SUBMIT = 'submit';
DropArea.EVENT_DRAG_OVER = 'drag_over';
DropArea.EVENT_DRAG_LEAVE = 'drag_leave';
DropArea.EVENT_DROP = 'drop';