/**
 * @author Vladimir Kozhin <affka@affka.ru>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _Component2 = require('../base/Component');

var _Component3 = _interopRequireDefault(_Component2);

var _FormElement = require('../form/FormElement');

var _FormElement2 = _interopRequireDefault(_FormElement);

var _InputElement = require('../form/InputElement');

var _InputElement2 = _interopRequireDefault(_InputElement);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Form = function (_Component) {
    _inherits(Form, _Component);

    function Form() {
        _classCallCheck(this, Form);

        return _possibleConstructorReturn(this, (Form.__proto__ || Object.getPrototypeOf(Form)).apply(this, arguments));
    }

    _createClass(Form, [{
        key: 'preInit',
        value: function preInit() {
            /**
             * @type {HTMLElement}
             */
            this.container = null;

            /**
             * @type {object}
             */
            this.inputAttributes = null;

            /**
             * @type {boolean}
             */
            this._isMultiple = true;

            /**
             * @type {FormElement}
             */
            this._formElement = null;

            /**
             * @type {InputElement}
             */
            this._lastInputElement = null;

            /**
             * @type {InputElement[]}
             */
            this._inputElements = [];

            _get(Form.prototype.__proto__ || Object.getPrototypeOf(Form.prototype), 'preInit', this).apply(this, arguments);
        }
    }, {
        key: 'init',
        value: function init() {
            // Init container
            this.container = this.container || document.body;

            // Create form element
            this._formElement = new _FormElement2.default();
            this._formElement.appendTo(this.container);

            // Create new input element
            this._refreshInput();
        }

        /**
         *
         * @returns {boolean}
         */

    }, {
        key: 'getMultiple',
        value: function getMultiple() {
            return this._isMultiple;
        }

        /**
         *
         * @param {boolean} value
         */

    }, {
        key: 'setMultiple',
        value: function setMultiple(value) {
            this._isMultiple = value;

            if (this._lastInputElement) {
                this._lastInputElement.element.multiple = value;
            }
        }
    }, {
        key: 'submit',
        value: function submit(url, target) {
            // Set destination
            this._formElement.element.action = url;
            this._formElement.element.target = target;

            this._formElement.element.submit();

            // Reset values
            this._formElement.element.action = '';
            this._formElement.element.target = '';
        }

        /**
         * Open browse files dialog on local machine
         */

    }, {
        key: 'browse',
        value: function browse() {
            var event = document.createEvent('MouseEvents');
            event.initEvent('click', true, false);

            this._lastInputElement.element.dispatchEvent(event);
        }

        /**
         *
         * @protected
         */

    }, {
        key: '_refreshInput',
        value: function _refreshInput() {
            // Freeze previous element, but do not detach
            if (this._lastInputElement) {
                this._lastInputElement.freeze();
                this.container.appendChild(this._lastInputElement.element);
            }

            this._lastInputElement = new _InputElement2.default({
                multiple: this.getMultiple(),
                onChange: this._onInputChange.bind(this),
                inputAttributes: this.inputAttributes
            });
            this._lastInputElement.appendTo(this._formElement.element);
            this._inputElements.push(this._lastInputElement);
        }

        /**
         *
         * @param {object} event
         * @protected
         */

    }, {
        key: '_onInputChange',
        value: function _onInputChange(event) {
            event = event || window.event;
            event.preventDefault();

            if (this._lastInputElement.getCount() === 0) {
                return;
            }

            var files = {};
            for (var i = 0, l = this._lastInputElement.getCount(); i < l; i++) {
                files[this._lastInputElement.getFilePath(i)] = this._lastInputElement.getFileNative(i) || {};
            }
            this.trigger(Form.EVENT_SUBMIT, [files]);

            this._refreshInput();
        }
    }, {
        key: 'destroy',
        value: function destroy() {
            if (this._formElement) {
                this._formElement.destroy();
            }
            for (var i = 0, l = this._inputElements.length; i < l; i++) {
                this._inputElements[i].destroy();
            }

            this.off(Form.EVENT_SUBMIT);
        }
    }]);

    return Form;
}(_Component3.default);

exports.default = Form;


Form.EVENT_SUBMIT = 'submit';