/**
 * @author Vladimir Kozhin <affka@affka.ru>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Exception = require('../base/Exception');

var _Exception2 = _interopRequireDefault(_Exception);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ClassHelper = function () {
    function ClassHelper() {
        _classCallCheck(this, ClassHelper);
    }

    _createClass(ClassHelper, null, [{
        key: 'generateUid',
        value: function generateUid() {
            return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                var r = Math.random() * 16 | 0,
                    v = c == 'x' ? r : r & 0x3 | 0x8;
                return v.toString(16);
            });
        }
    }, {
        key: 'createObject',
        value: function createObject(config) {
            if (!config.className) {
                throw new _Exception2.default('Wrong configuration for create object.');
            }

            config = this.clone(config);
            var className = config.className;
            delete config.className;

            // Get class
            var objectClass = className;
            if (typeof objectClass !== 'function') {
                throw new _Exception2.default('Not found class `' + className + '` for create instance.');
            }

            return new objectClass(config);
        }

        /**
         * @param {object} object
         * @param {object} config
         */

    }, {
        key: 'configure',
        value: function configure(object, config) {
            for (var key in config) {
                if (!config.hasOwnProperty(key)) {
                    continue;
                }

                // Generate setter name
                var setter = 'set' + key.charAt(0).toUpperCase() + key.slice(1);

                if (typeof object[setter] !== 'function') {
                    if (typeof object[key] === 'function') {
                        throw new _Exception2.default('You can not replace from config function `' + key + '` in object `' + object.constructor.name + '`.');
                    }

                    if (typeof object[key] === 'undefined') {
                        throw new _Exception2.default('Config param `' + key + '` is undefined in object `' + object.constructor.name + '`.');
                    }
                }

                if (typeof object[key] !== 'undefined' && typeof object[key] !== 'function') {
                    if (this._isSimpleObject(object[key]) && this._isSimpleObject(config[key])) {
                        object[key] = this.merge(object[key], config[key]);
                    } else {
                        object[key] = config[key];
                    }
                } else if (typeof object[setter] === 'function') {
                    object[setter].call(object, config[key]);
                }
            }
        }

        /**
         * @param {object} [obj]
         * @returns {object}
         */

    }, {
        key: 'merge',
        value: function merge(obj) {
            var dst = {};

            for (var i = 0, l = arguments.length; i < l; ++i) {
                obj = arguments[i];
                if ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) !== 'object' || obj instanceof Array) {
                    continue;
                }

                for (var key in obj) {
                    if (obj.hasOwnProperty(key)) {
                        if (this._isSimpleObject(obj[key])) {
                            dst[key] = this.merge(dst[key], obj[key]);
                        } else {
                            dst[key] = obj[key];
                        }
                    }
                }
            }

            return dst;
        }

        /**
         *
         * @param {object} obj
         * @returns {object}
         */

    }, {
        key: 'clone',
        value: function clone(obj) {
            var clone = {};
            for (var key in obj) {
                if (obj.hasOwnProperty(key)) {
                    clone[key] = obj[key];
                }
            }
            return clone;
        }
    }, {
        key: '_isSimpleObject',
        value: function _isSimpleObject(obj) {
            return obj && (typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' && !(obj instanceof Array) && obj.constructor === Object;
        }
    }]);

    return ClassHelper;
}();

exports.default = ClassHelper;