/**
 * @author Vladimir Kozhin <affka@affka.ru>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Form = require('./form/Form');

var _Form2 = _interopRequireDefault(_Form);

var _DropArea = require('./form/DropArea');

var _DropArea2 = _interopRequireDefault(_DropArea);

var _BrowserHelper = require('./helpers/BrowserHelper');

var _BrowserHelper2 = _interopRequireDefault(_BrowserHelper);

var _ClassHelper = require('./helpers/ClassHelper');

var _ClassHelper2 = _interopRequireDefault(_ClassHelper);

var _QueueCollection = require('./models/QueueCollection');

var _QueueCollection2 = _interopRequireDefault(_QueueCollection);

var _QueueManager = require('./managers/QueueManager');

var _QueueManager2 = _interopRequireDefault(_QueueManager);

var _File = require('./models/File');

var _File2 = _interopRequireDefault(_File);

var _IframeUploader = require('./uploaders/IframeUploader');

var _IframeUploader2 = _interopRequireDefault(_IframeUploader);

var _XhrUploader = require('./uploaders/XhrUploader');

var _XhrUploader2 = _interopRequireDefault(_XhrUploader);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var FileUp = function () {
    function FileUp(config) {
        _classCallCheck(this, FileUp);

        /**
         * @type {string}
         */
        this.backendUrl = null;

        /**
         * @type {object}
         */
        this.backendParams = {};

        /**
         * @type {Form}
         */
        this.form = {
            className: _Form2.default
        };

        /**
         * @type {DropArea}
         */
        this.dropArea = {
            className: _DropArea2.default
        };

        /**
         * @type {QueueCollection}
         */
        this.queue = {
            className: _QueueCollection2.default
        };

        /**
         * @type {QueueManager}
         */
        this.queueManager = {
            className: _QueueManager2.default
        };

        /**
         * @type {File}
         */
        this.fileConfig = {
            className: _File2.default
        };

        /**
         * Set force iframe or xhr. Null - auto detect from browser support
         * @type {string|null}
         */
        this.uploaderName = null;

        this.uploaderConfigs = {
            iframe: {
                className: _IframeUploader2.default
            },
            xhr: {
                className: _XhrUploader2.default
            }
        };

        if ((typeof config === 'undefined' ? 'undefined' : _typeof(config)) === 'object') {
            _ClassHelper2.default.configure(this, config);
        }

        this._initForm();
        this._initDropArea();
        this._initQueue();
        this._initManagers();
    }

    _createClass(FileUp, [{
        key: '_initForm',
        value: function _initForm() {
            this.form = _ClassHelper2.default.createObject(this.form);
            this.form.on(_Form2.default.EVENT_SUBMIT, this._onFormSubmit.bind(this));
        }
    }, {
        key: '_initDropArea',
        value: function _initDropArea() {
            this.dropArea = _ClassHelper2.default.createObject(this.dropArea);
            this.dropArea.on(_DropArea2.default.EVENT_SUBMIT, this._onFormSubmit.bind(this));
        }
    }, {
        key: '_initQueue',
        value: function _initQueue() {
            this.queue = _ClassHelper2.default.createObject(this.queue);
        }
    }, {
        key: '_initManagers',
        value: function _initManagers() {
            var managers = ['queue'];

            for (var i = 0, l = managers.length; i < l; i++) {
                var name = managers[i] + 'Manager';
                this[name] = _ClassHelper2.default.createObject(_ClassHelper2.default.merge({
                    collection: this.queue
                }, this[name]));
            }
        }

        /**
         * Open browse files dialog on local machine
         */

    }, {
        key: 'browse',
        value: function browse() {
            this.form.browse();
        }

        /**
         *
         */

    }, {
        key: 'destroy',
        value: function destroy() {
            this.form.destroy();
        }

        /**
         *
         * @param {object} nativeFiles
         * @protected
         */

    }, {
        key: '_onFormSubmit',
        value: function _onFormSubmit(nativeFiles) {
            var uploader = null;
            var isIE = _BrowserHelper2.default.isIE();
            if (this.uploaderName === 'iframe' || isIE && isIE < 10) {
                uploader = _ClassHelper2.default.createObject(_ClassHelper2.default.merge({
                    url: this.backendUrl,
                    params: this.backendParams,
                    form: this.form
                }, this.uploaderConfigs.iframe));
            }

            var queueTotal = this.queue.getCount();

            var i = 0;
            var files = [];
            for (var path in nativeFiles) {
                if (nativeFiles.hasOwnProperty(path)) {
                    // Files limit
                    queueTotal++;
                    if (this.queue.maxFiles !== null && queueTotal > this.queue.maxFiles) {
                        break;
                    }

                    var nativeFile = nativeFiles[path];
                    var file = _ClassHelper2.default.createObject(_ClassHelper2.default.merge({
                        index: i++,
                        native: nativeFile,
                        path: path,
                        type: nativeFile.type || '',
                        bytesTotal: nativeFile.fileSize || nativeFile.size || 0
                    }, this.fileConfig));

                    file.setUploader(uploader || _ClassHelper2.default.createObject(_ClassHelper2.default.merge({
                        url: this.backendUrl,
                        params: this.backendParams,
                        file: file
                    }, this.uploaderConfigs.xhr)));

                    files.push(file);
                }
            }

            if (uploader instanceof _IframeUploader2.default) {
                uploader.files = files;
            }

            this.queue.add(files);
        }
    }]);

    return FileUp;
}();

exports.default = FileUp;