/**
 * @author Vladimir Kozhin <affka@affka.ru>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _Component2 = require('../base/Component');

var _Component3 = _interopRequireDefault(_Component2);

var _FileProgress = require('./FileProgress');

var _FileProgress2 = _interopRequireDefault(_FileProgress);

var _ClassHelper = require('../helpers/ClassHelper');

var _ClassHelper2 = _interopRequireDefault(_ClassHelper);

var _BaseUploader = require('../uploaders/BaseUploader');

var _BaseUploader2 = _interopRequireDefault(_BaseUploader);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var File = function (_Component) {
    _inherits(File, _Component);

    function File() {
        _classCallCheck(this, File);

        return _possibleConstructorReturn(this, (File.__proto__ || Object.getPrototypeOf(File)).apply(this, arguments));
    }

    _createClass(File, [{
        key: 'preInit',
        value: function preInit() {

            this.index = 0;

            /**
             * @type {string}
             */
            this._uid = null;

            /**
             * @type {File}
             */
            this._native = null;

            /**
             * @type {FileProgress}
             */
            this.progress = {
                className: _FileProgress2.default
            };

            /**
             * @type {BaseUploader}
             */
            this._uploader = null;

            /**
             * @type {string}
             */
            this._path = '';

            /**
             * @type {string}
             */
            this._type = '';

            /**
             * @type {number}
             */
            this._bytesUploaded = 0;

            /**
             * @type {number}
             */
            this._bytesUploadEnd = 0;

            /**
             * @type {number}
             */
            this._bytesTotal = 0;

            /**
             * @type {string}
             */
            this._status = 'queue';

            /**
             * @type {string|null}
             */
            this._result = null;

            /**
             * @type {number|null}
             */
            this._resultHttpStatus = null;

            /**
             * @type {string|null}
             */
            this._resultHttpMessage = null;

            _get(File.prototype.__proto__ || Object.getPrototypeOf(File.prototype), 'preInit', this).apply(this, arguments);
        }
    }, {
        key: 'init',
        value: function init() {
            this._uid = this._uid || _ClassHelper2.default.generateUid();
            this.progress = _ClassHelper2.default.createObject(_ClassHelper2.default.merge({
                file: this
            }, this.progress));
        }
    }, {
        key: 'start',
        value: function start() {
            this._uploader.start();
        }
    }, {
        key: 'pause',
        value: function pause() {
            if (this._status === File.STATUS_PAUSE) {
                this.start();
                return;
            }

            this.stop();
            this.setStatus(File.STATUS_PAUSE);
        }
    }, {
        key: 'stop',
        value: function stop() {
            this._uploader.stop();
        }

        /**
         *
         * @param {string} value
         */

    }, {
        key: 'setUid',
        value: function setUid(value) {
            this._uid = value;
        }

        /**
         *
         * @returns {string}
         */

    }, {
        key: 'getUid',
        value: function getUid() {
            return this._uid;
        }

        /**
         *
         * @param {File} value
         */

    }, {
        key: 'setNative',
        value: function setNative(value) {
            this._native = value;
        }

        /**
         *
         * @returns {File}
         */

    }, {
        key: 'getNative',
        value: function getNative() {
            return this._native;
        }

        /**
         *
         * @param {string} value
         */

    }, {
        key: 'setPath',
        value: function setPath(value) {
            this._path = value;
        }

        /**
         *
         * @returns {string}
         */

    }, {
        key: 'getPath',
        value: function getPath() {
            return this._path;
        }

        /**
         *
         * @param {string} value
         */

    }, {
        key: 'setType',
        value: function setType(value) {
            this._type = value;
        }

        /**
         *
         * @returns {string}
         */

    }, {
        key: 'getType',
        value: function getType() {
            return this._type;
        }

        /**
         *
         * @returns {string}
         */

    }, {
        key: 'getName',
        value: function getName() {
            var path = this.getPath();
            var matches = /[^\/\\]+$/.exec(path);

            return matches ? matches[0].replace(/^([^?]+).*$/, '$1') : path;
        }

        /**
         *
         * @param {BaseUploader} value
         */

    }, {
        key: 'setUploader',
        value: function setUploader(value) {
            var _this2 = this;

            if (this._uploader) {
                this._uploader.stop();
            }

            this._uploader = value;

            this._uploader.on(_BaseUploader2.default.EVENT_START, function () {
                _this2.progress.reset();
                _this2.trigger(File.EVENT_PROGRESS, [_this2]);

                _this2._resultHttpStatus = null;
                _this2._resultHttpMessage = null;
                _this2.setStatus(File.STATUS_PROCESS);
            });

            this._uploader.on(_BaseUploader2.default.EVENT_ERROR, function () {
                _this2.progress.reset();
                _this2.trigger(File.EVENT_PROGRESS, [_this2]);

                _this2._result = File.RESULT_ERROR;
                _this2.setStatus(File.STATUS_END);
            });

            this._uploader.on(_BaseUploader2.default.EVENT_END, function () {
                _this2.setBytesUploaded(_this2.getBytesTotal());
                _this2.progress.reset();
                _this2.trigger(File.EVENT_PROGRESS, [_this2]);

                _this2._result = File.RESULT_SUCCESS;
                _this2.setStatus(File.STATUS_END);
            });

            this._uploader.on(_BaseUploader2.default.EVENT_PROGRESS, function (bytesUploaded) {
                _this2.progress.add(bytesUploaded);
                _this2.setBytesUploaded(bytesUploaded);
            });
        }

        /**
         *
         * @returns {BaseUploader}
         */

    }, {
        key: 'getUploader',
        value: function getUploader() {
            return this._uploader;
        }

        /**
         *
         * @param {number} value
         */

    }, {
        key: 'setBytesUploaded',
        value: function setBytesUploaded(value) {
            if (this._bytesUploaded === value) {
                return;
            }

            this._bytesUploaded = value;
            this.trigger(File.EVENT_PROGRESS, [this]);
        }

        /**
         *
         * @returns {number}
         */

    }, {
        key: 'getBytesUploaded',
        value: function getBytesUploaded() {
            return this._bytesUploaded;
        }

        /**
         *
         * @param {number} value
         */

    }, {
        key: 'setBytesUploadEnd',
        value: function setBytesUploadEnd(value) {
            if (this._bytesUploadEnd === value) {
                return;
            }

            this._bytesUploadEnd = value;
            this.trigger(File.EVENT_PROGRESS, [this]);
        }

        /**
         *
         * @returns {number}
         */

    }, {
        key: 'getBytesUploadEnd',
        value: function getBytesUploadEnd() {
            return this._bytesUploadEnd;
        }

        /**
         *
         * @param {number} value
         */

    }, {
        key: 'setBytesTotal',
        value: function setBytesTotal(value) {
            if (this._bytesTotal === value) {
                return;
            }

            this._bytesTotal = value;
            this.trigger(File.EVENT_PROGRESS, [this]);
        }

        /**
         *
         * @returns {number}
         */

    }, {
        key: 'getBytesTotal',
        value: function getBytesTotal() {
            return this._bytesTotal;
        }

        /**
         *
         * @param {string} value
         */

    }, {
        key: 'setResult',
        value: function setResult(value) {
            this._result = value;
        }

        /**
         *
         * @returns {string}
         */

    }, {
        key: 'getResult',
        value: function getResult() {
            return this._result;
        }

        /**
         *
         * @returns {boolean}
         */

    }, {
        key: 'isResultSuccess',
        value: function isResultSuccess() {
            return this._result === File.RESULT_SUCCESS;
        }

        /**
         *
         * @returns {boolean}
         */

    }, {
        key: 'isResultError',
        value: function isResultError() {
            return this._result === File.RESULT_ERROR;
        }

        /**
         *
         * @param {string} value
         */

    }, {
        key: 'setResultHttpStatus',
        value: function setResultHttpStatus(value) {
            this._resultHttpStatus = value;
        }

        /**
         *
         * @returns {number|null}
         */

    }, {
        key: 'getResultHttpStatus',
        value: function getResultHttpStatus() {
            return this._resultHttpStatus;
        }

        /**
         *
         * @param {string} value
         */

    }, {
        key: 'setResultHttpMessage',
        value: function setResultHttpMessage(value) {
            this._resultHttpMessage = value;
        }

        /**
         *
         * @returns {string|null}
         */

    }, {
        key: 'getResultHttpMessage',
        value: function getResultHttpMessage() {
            return this._resultHttpMessage;
        }

        /**
         *
         * @returns {boolean}
         */

    }, {
        key: 'isStatusQueue',
        value: function isStatusQueue() {
            return this._status === File.STATUS_QUEUE;
        }

        /**
         *
         * @returns {boolean}
         */

    }, {
        key: 'isStatusProcess',
        value: function isStatusProcess() {
            return this._status === File.STATUS_PROCESS;
        }

        /**
         *
         * @returns {boolean}
         */

    }, {
        key: 'isStatusPause',
        value: function isStatusPause() {
            return this._status === File.STATUS_PAUSE;
        }

        /**
         *
         * @returns {boolean}
         */

    }, {
        key: 'isStatusEnd',
        value: function isStatusEnd() {
            return this._status === File.STATUS_END;
        }

        /**
         *
         * @returns {string}
         */

    }, {
        key: 'getStatus',
        value: function getStatus() {
            return this._status;
        }

        /**
         *
         * @returns {{path: string, type: string, bytesUploaded: number, bytesUploadEnd: number, bytesTotal: number, status: string, result: (string|null), resultHttpStatus: (number|null), resultHttpMessage: (string|null)}}
         */

    }, {
        key: 'toJSON',
        value: function toJSON() {
            return {
                uid: this.getUid(),
                name: this.getName(),
                path: this.getPath(),
                type: this.getType(),
                bytesUploaded: this.getBytesUploaded(),
                bytesUploadEnd: this.getBytesUploadEnd(),
                bytesTotal: this.getBytesTotal(),
                status: this.getStatus(),
                result: this.getResult(),
                resultHttpStatus: this.getResultHttpStatus(),
                resultHttpMessage: this.getResultHttpMessage(),
                progress: this.progress.toJSON()
            };
        }
    }, {
        key: 'setStatus',
        value: function setStatus(value) {
            if (this._status === value) {
                return;
            }

            this._status = value;
            this.trigger(File.EVENT_STATUS, [this, this._status]);
        }
    }]);

    return File;
}(_Component3.default);

exports.default = File;


File.STATUS_QUEUE = 'queue';
File.STATUS_PROCESS = 'process';
File.STATUS_PAUSE = 'pause';
File.STATUS_END = 'end';
File.RESULT_SUCCESS = 'success';
File.RESULT_ERROR = 'error';

File.EVENT_STATUS = 'status';
File.EVENT_PROGRESS = 'progress';