/**
 * @author Vladimir Kozhin <affka@affka.ru>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _BaseObject2 = require('../base/BaseObject');

var _BaseObject3 = _interopRequireDefault(_BaseObject2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FileProgress = function (_BaseObject) {
    _inherits(FileProgress, _BaseObject);

    function FileProgress() {
        _classCallCheck(this, FileProgress);

        return _possibleConstructorReturn(this, (FileProgress.__proto__ || Object.getPrototypeOf(FileProgress)).apply(this, arguments));
    }

    _createClass(FileProgress, [{
        key: 'preInit',
        value: function preInit() {

            /**
             * @type {number}
             */
            this.speedMinMeasurement = 2;

            /**
             * @type {number}
             */
            this.speedMaxMeasurement = 5;

            /**
             * @type {File}
             */
            this.file = null;

            this.history = [];

            this._lastTime = null;

            _get(FileProgress.prototype.__proto__ || Object.getPrototypeOf(FileProgress.prototype), 'preInit', this).apply(this, arguments);
        }
    }, {
        key: 'add',
        value: function add(bytesUploaded) {
            var now = new Date().getTime();

            this.history.push({
                bytes: bytesUploaded - this.file.getBytesUploaded(),
                duration: this._lastTime ? now - this._lastTime : null
            });
            this._lastTime = now;
        }
    }, {
        key: 'reset',
        value: function reset() {
            this.history = [];
            this._lastTime = null;
        }
    }, {
        key: 'toJSON',
        value: function toJSON() {
            return {
                history: this.history,
                timeLeft: this.getTimeLeft(),
                speed: this.getSpeed(),
                percent: this.getPercent()
            };
        }

        /**
         * @returns {number} Seconds
         */

    }, {
        key: 'getTimeLeft',
        value: function getTimeLeft() {
            var bytesTotal = this.file.getBytesTotal();
            if (bytesTotal === 0) {
                return 0;
            }

            var speed = this.getSpeed();
            if (speed === 0) {
                return 0;
            }

            var bytesUploaded = this.file.getBytesUploaded();

            return Math.ceil((bytesTotal - bytesUploaded) / speed);
        }

        /**
         * @returns {number} Bytes in second
         */

    }, {
        key: 'getSpeed',
        value: function getSpeed() {
            if (this.history.length < this.speedMinMeasurement) {
                return 0;
            }

            // Get last diff values
            var history = this.history.slice(-1 * this.speedMaxMeasurement);

            // Calculate average upload speed
            var summaryBytes = 0;
            var summaryDuration = 0;
            for (var i = 0, l = history.length; i < l; i++) {
                summaryBytes += history[i].bytes;
                summaryDuration += history[i].duration;
            }

            if (summaryBytes === 0 || summaryDuration === 0) {
                return 0;
            }

            return Math.round(summaryBytes / (summaryDuration / 1000));
        }

        /**
         * @returns {number}
         */

    }, {
        key: 'getPercent',
        value: function getPercent() {
            var bytesTotal = this.file.getBytesTotal();
            if (bytesTotal === 0) {
                return 0;
            }

            var bytesUploaded = this.file.getBytesUploaded();
            return Math.round(bytesUploaded * 100 / bytesTotal);
        }
    }]);

    return FileProgress;
}(_BaseObject3.default);

exports.default = FileProgress;