/**
 * @author Vladimir Kozhin <affka@affka.ru>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _Component2 = require('../base/Component');

var _Component3 = _interopRequireDefault(_Component2);

var _File = require('./File');

var _File2 = _interopRequireDefault(_File);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var QueueCollection = function (_Component) {
    _inherits(QueueCollection, _Component);

    function QueueCollection() {
        _classCallCheck(this, QueueCollection);

        return _possibleConstructorReturn(this, (QueueCollection.__proto__ || Object.getPrototypeOf(QueueCollection)).apply(this, arguments));
    }

    _createClass(QueueCollection, [{
        key: 'preInit',
        value: function preInit() {

            /**
             * @type {number}
             */
            this.maxConcurrentUploads = 3;

            /**
             * @type {number|null}
             */
            this.maxFiles = null;

            /**
             * @type {File[]}
             */
            this._files = [];

            this._onStatusChange = this._onStatusChange.bind(this);
            this._onProgressChange = this._onProgressChange.bind(this);

            _get(QueueCollection.prototype.__proto__ || Object.getPrototypeOf(QueueCollection.prototype), 'preInit', this).apply(this, arguments);
        }

        /**
         *
         * @param {File[]} files
         */

    }, {
        key: 'add',
        value: function add(files) {
            this._files = this._files.concat(files);

            for (var i = 0, l = files.length; i < l; i++) {
                files[i].on(_File2.default.EVENT_STATUS, this._onStatusChange);
                files[i].on(_File2.default.EVENT_PROGRESS, this._onProgressChange);
            }

            this.trigger(QueueCollection.EVENT_ADD, [files]);
        }

        /**
         *
         * @param {File[]} files
         */

    }, {
        key: 'remove',
        value: function remove(files) {
            for (var i = 0, l = files.length; i < l; i++) {
                var index = this._files.indexOf(files[i]);
                if (index !== -1) {
                    this._files.splice(index, 1);
                }
            }

            for (var i2 = 0, l2 = files.length; i2 < l2; i2++) {
                files[i2].off(_File2.default.EVENT_STATUS, this._onStatusChange);
                files[i2].off(_File2.default.EVENT_PROGRESS, this._onProgressChange);
            }

            this.trigger(QueueCollection.EVENT_REMOVE, [files]);
        }

        /**
         * @returns {File[]}
         */

    }, {
        key: 'getFiles',
        value: function getFiles() {
            return this._files;
        }

        /**
         *
         * @param {string} uid
         * @returns {File|null}
         */

    }, {
        key: 'getByUid',
        value: function getByUid(uid) {
            for (var i = 0, l = this._files.length; i < l; i++) {
                if (this._files[i].getUid() === uid) {
                    return this._files[i];
                }
            }
            return null;
        }

        /**
         * @returns {number}
         */

    }, {
        key: 'getCount',
        value: function getCount() {
            return this._files.length;
        }

        /**
         *
         * @param {string} status
         * @returns {number}
         */

    }, {
        key: 'getCountByStatus',
        value: function getCountByStatus(status) {
            var iCount = 0;
            for (var i = 0, l = this._files.length; i < l; i++) {
                if (this._files[i].getStatus() === status) {
                    iCount++;
                }
            }
            return iCount;
        }

        /**
         * Search file for next uploading
         * @returns {File|null}
         */

    }, {
        key: 'getNextForUpload',
        value: function getNextForUpload() {
            if (this.getCountByStatus(_File2.default.STATUS_PROCESS) >= this.maxConcurrentUploads) {
                return null;
            }

            for (var i = 0, l = this._files.length; i < l; i++) {
                if (this._files[i].isStatusQueue()) {
                    return this._files[i];
                }
            }

            return null;
        }

        /**
         *
         * @param {File} file
         * @protected
         */

    }, {
        key: '_onStatusChange',
        value: function _onStatusChange(file) {
            this.trigger(QueueCollection.EVENT_ITEM_STATUS, [file]);

            if (file.isStatusEnd()) {
                this.trigger(QueueCollection.EVENT_ITEM_END, [file]);

                if (this.getCount() === this.getCountByStatus(_File2.default.STATUS_END)) {
                    this.trigger(QueueCollection.EVENT_ALL_END, [this._files]);
                }
            }
        }

        /**
         *
         * @param {File} file
         * @protected
         */

    }, {
        key: '_onProgressChange',
        value: function _onProgressChange(file) {
            this.trigger(QueueCollection.EVENT_ITEM_PROGRESS, [file]);
        }
    }]);

    return QueueCollection;
}(_Component3.default);

exports.default = QueueCollection;


QueueCollection.EVENT_ADD = 'add';
QueueCollection.EVENT_REMOVE = 'remove';
QueueCollection.EVENT_ALL_END = 'all_end';
QueueCollection.EVENT_ITEM_STATUS = 'item_status';
QueueCollection.EVENT_ITEM_PROGRESS = 'item_progress';
QueueCollection.EVENT_ITEM_END = 'item_end';