/**
 * @author Vladimir Kozhin <affka@affka.ru>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _BaseUploader2 = require('./BaseUploader');

var _BaseUploader3 = _interopRequireDefault(_BaseUploader2);

var _BrowserHelper = require('../helpers/BrowserHelper');

var _BrowserHelper2 = _interopRequireDefault(_BrowserHelper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var IframeUploader = function (_BaseUploader) {
    _inherits(IframeUploader, _BaseUploader);

    function IframeUploader() {
        _classCallCheck(this, IframeUploader);

        return _possibleConstructorReturn(this, (IframeUploader.__proto__ || Object.getPrototypeOf(IframeUploader)).apply(this, arguments));
    }

    _createClass(IframeUploader, [{
        key: 'preInit',
        value: function preInit() {

            /**
             * @type {File[]}
             */
            this.files = null;

            /**
             * @type {Form}
             */
            this.form = null;

            /**
             * @type {HTMLElement}
             */
            this.container = null;

            /**
             * @type {string}
             */
            this.namePrefix = 'FileUpIframe';

            /**
             * @type {string}
             */
            this._name = '';

            /**
             * @type {HTMLElement}
             */
            this._wrapper = null;

            /**
             * @type {HTMLElement}
             */
            this._frame = null;

            /**
             * @type {number|null}
             */
            this._frameLoadTimer = null;

            /**
             * @type {boolean}
             */
            this._isFrameLoaded = false;

            _get(IframeUploader.prototype.__proto__ || Object.getPrototypeOf(IframeUploader.prototype), 'preInit', this).apply(this, arguments);
        }
    }, {
        key: 'init',
        value: function init() {
            // Generate name
            this._name = this.namePrefix + ++IframeUploader._Counter;

            // Init container
            this.container = this.container || document.body;

            // Render frame
            this._initContainer();
            this._initFrame();
        }
    }, {
        key: 'start',
        value: function start() {
            // Start upload
            this.trigger(_BaseUploader3.default.EVENT_START);
            this.form.submit(this.getUrl(), this._name);
        }
    }, {
        key: 'stop',
        value: function stop() {
            this._clearTimer();

            if (this._frame) {
                this._frame.onload = null;
                this._frame.onreadystatechange = null;

                this._wrapper.removeChild(this._frame);
                delete this._frame;
            }

            _get(IframeUploader.prototype.__proto__ || Object.getPrototypeOf(IframeUploader.prototype), 'start', this).call(this);
        }
    }, {
        key: 'getUrl',
        value: function getUrl() {
            var uids = [];
            for (var key in this.files) {
                if (this.files.hasOwnProperty(key)) {
                    uids.push(this.files[key].getUid());
                }
            }

            var params = this.getParams();
            params.uids = uids;

            return this.constructor.addUrlParams(this._url, params);
        }
    }, {
        key: '_initContainer',
        value: function _initContainer() {
            this._wrapper = document.createElement('div');
            this._wrapper.style.position = 'absolute';
            this._wrapper.style.width = 0;
            this._wrapper.style.height = 0;
            this._wrapper.style.top = '-100px';
            this._wrapper.style.left = '-100px';
            this._wrapper.style.display = 'none';

            this.container.appendChild(this._wrapper);
        }
    }, {
        key: '_initFrame',
        value: function _initFrame() {
            var isCreated = false;
            var isIE = _BrowserHelper2.default.isIE();

            if (isIE && isIE < 10) {
                try {
                    this._frame = document.createElement('<iframe name="' + this._name + '">');
                    isCreated = true;
                } catch (e) {
                    // It seems IE9 in compatability mode.
                }
            }

            if (!isCreated) {
                this._frame = document.createElement('iframe');
                this._frame.name = this._name;
            }

            this._frame.src = 'javascript:{};';
            this._wrapper.appendChild(this._frame);

            // Subscribe on iframe load events
            this._frame.onreadystatechange = this._onReadyStateChange.bind(this);
            this._frame.onload = this._onLoad.bind(this);
        }
    }, {
        key: '_onReadyStateChange',
        value: function _onReadyStateChange(event) {
            var _this2 = this;

            switch (this._frame.readyState) {
                case 'complete':
                case 'interactive':
                    this._clearTimer();
                    this._frameLoadTimer = setTimeout(function () {
                        try {
                            _this2._frame.contentWindow.document.body;
                            _this2._onLoad(event);
                        } catch (e) {
                            _this2._onReadyStateChange(event);
                        }
                    }, 1000);
                    break;
            }
        }
    }, {
        key: '_onLoad',
        value: function _onLoad() {
            this._clearTimer();

            // Check already loaded
            if (this._isFrameLoaded) {
                return;
            }
            this._isFrameLoaded = true;

            var document = null;
            var status = null;
            var errorMessage = '';

            // Catch iframe load error in Firefox.
            try {
                document = this._frame.contentWindow.document;
                document.body;
            } catch (e) {
                status = 403;
                errorMessage = e.toString();
            }

            var text = document.body.innerText || document.body.innerHTML;
            if (!status) {
                status = 201; // Created
            }

            if (status >= 200 && status < 300) {
                var data = (this.responseParser || this._defaultResponseParser).call(this, text);
                if (data instanceof Array) {
                    for (var i = 0, l = this.files.length; i < l; i++) {
                        this.files[i].setResultHttpStatus(status);
                        this.files[i].setResultHttpMessage(data[i]);
                    }
                    this.trigger(_BaseUploader3.default.EVENT_END, [status, data]);
                } else {
                    for (var i = 0, l = this.files.length; i < l; i++) {
                        this.files[i].setResultHttpStatus(status);
                        this.files[i].setResultHttpMessage(data);
                    }
                    this.trigger(_BaseUploader3.default.EVENT_END, [500, data]);
                }
            } else {
                for (var i = 0, l = this.files.length; i < l; i++) {
                    this.files[i].setResultHttpStatus(status);
                    this.files[i].setResultHttpMessage(errorMessage);
                }
                this.trigger(_BaseUploader3.default.EVENT_ERROR, [status, errorMessage]);
            }

            this.stop();
        }
    }, {
        key: '_clearTimer',
        value: function _clearTimer() {
            if (this._frameLoadTimer) {
                clearTimeout(this._frameLoadTimer);
            }
        }
    }]);

    return IframeUploader;
}(_BaseUploader3.default);

exports.default = IframeUploader;


IframeUploader._Counter = 0;