/**
* @author Vladimir Kozhin <affka@affka.ru>
* @license MIT
*/

'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _BaseUploader2 = require('./BaseUploader');

var _BaseUploader3 = _interopRequireDefault(_BaseUploader2);

var _BrowserHelper = require('../helpers/BrowserHelper');

var _BrowserHelper2 = _interopRequireDefault(_BrowserHelper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var XhrUploader = function (_BaseUploader) {
    _inherits(XhrUploader, _BaseUploader);

    function XhrUploader() {
        _classCallCheck(this, XhrUploader);

        return _possibleConstructorReturn(this, (XhrUploader.__proto__ || Object.getPrototypeOf(XhrUploader)).apply(this, arguments));
    }

    _createClass(XhrUploader, [{
        key: 'preInit',
        value: function preInit() {
            /**
             * @type {string}
             */
            this.method = 'PUT';

            /**
             * @type {File}
             */
            this.file = null;

            /**
             * @type {number}
             */
            this.minProgressUpdateIntervalMs = 500;

            /**
             * This is IIS max httpRuntime@maxRequestLength value which is 2147482624 Kb
             * @type {number}
             */
            this.bytesMaxPart = 2097151 * 1024;

            /**
             * @type {object}
             */
            this.headers = {};

            /**
             * @type {number}
             */
            this._lastReportTime = null;

            /**
             * @type {XMLHttpRequest}
             */
            this._xhr = null;

            /**
             * @type {number}
             */
            this._bytesStart = 0;

            /**
             * @type {number|null}
             */
            this._bytesEnd = null;

            _get(XhrUploader.prototype.__proto__ || Object.getPrototypeOf(XhrUploader.prototype), 'preInit', this).apply(this, arguments);
        }
    }, {
        key: 'start',
        value: function start() {
            this._initXhr();
            this._startInternal();
        }
    }, {
        key: 'stop',
        value: function stop() {
            if (this._xhr) {
                if (this._xhr.upload) {
                    this._xhr.upload.onprogress = null;
                }
                this._xhr.onreadystatechange = null;
                this._xhr.abort();
            }

            _get(XhrUploader.prototype.__proto__ || Object.getPrototypeOf(XhrUploader.prototype), 'start', this).call(this);
        }
    }, {
        key: 'getUrl',
        value: function getUrl() {
            var params = this.getParams();
            params.uids = [this.file.getUid()];

            return this.constructor.addUrlParams(this._url, params);
        }

        /**
         * Create XHR object and subscribe on it events
         * @private
         */

    }, {
        key: '_initXhr',
        value: function _initXhr() {
            var _this2 = this;

            this._xhr = new XMLHttpRequest();
            this._xhr.upload.onprogress = this._onProgress.bind(this);
            this._xhr.onreadystatechange = this._onReadyStateChange.bind(this);
            this._xhr.open(this.method, this.getUrl(), true);

            try {
                this._xhr.withCredentials = true;
            } catch (e) {}

            // Set headers
            var headers = {};
            if (_BrowserHelper2.default.isWebkit() || _BrowserHelper2.default.isTrident()) {
                headers = _extends({}, headers, {
                    'If-None-Match': '*',
                    'If-Modified-Since': 'Mon, 26 Jul 1997 05:00:00 GMT',
                    'Cache-Control': 'no-cache',
                    'X-Requested-With': 'XMLHttpRequest'
                });
            }
            headers = _extends({}, headers, this.headers);
            Object.keys(headers).forEach(function (key) {
                _this2._xhr.setRequestHeader(key, headers[key]);
            });
        }
    }, {
        key: '_startInternal',
        value: function _startInternal() {
            this.trigger(_BaseUploader3.default.EVENT_START);

            // Set file name
            this._xhr.setRequestHeader('Content-Disposition', 'attachment; filename="' + encodeURI(this.file.getName()) + '"');

            var isFF = _BrowserHelper2.default.isFirefox();
            if (isFF && isFF < 7) {
                this._xhr.sendAsBinary(this.file.getNative().getAsBinary());
                return;
            }

            var bytesTotal = this.file.getBytesTotal();

            this._bytesStart = this.file.getBytesUploaded();
            this._bytesEnd = Math.min(this._bytesStart + this.bytesMaxPart, bytesTotal);

            // Check partial upload
            if (this._bytesStart > 0 || this._bytesEnd < bytesTotal) {
                this._xhr.setRequestHeader('Content-Range', 'bytes ' + this._bytesStart + '-' + (this._bytesEnd - 1) + '/' + bytesTotal);

                if (this._bytesEnd < bytesTotal) {
                    this._xhr.send(this.file.getNative().slice(this._bytesStart, this._bytesEnd));
                } else {
                    this._xhr.send(this.file.getNative().slice(this._bytesStart));
                }
            } else {
                this._xhr.send(this.file.getNative());
            }
        }

        /**
         *
         * @param {object} event
         * @protected
         */

    }, {
        key: '_onProgress',
        value: function _onProgress(event) {
            var iNow = new Date().getTime();
            if (this._lastReportTime && iNow - this._lastReportTime < this.minProgressUpdateIntervalMs) {
                return;
            }
            this._lastReportTime = iNow;

            var bytesUploaded = this._bytesStart + event.loaded;
            this.trigger(_BaseUploader3.default.EVENT_PROGRESS, [bytesUploaded]);
        }

        /**
         *
         * @protected
         */

    }, {
        key: '_onReadyStateChange',
        value: function _onReadyStateChange() {
            if (this._xhr.readyState !== 4) {
                return;
            }

            var text = this._xhr.responseText || this._xhr.statusText;
            this.file.setResultHttpStatus(this._xhr.status);

            if (this._xhr.status >= 200 && this._xhr.status < 300) {
                if (this._bytesEnd < this.file.getBytesTotal()) {
                    this.file.setBytesUploaded(this._bytesEnd);
                    this.stop();
                    this.start();

                    this.trigger(_BaseUploader3.default.EVENT_END_PART);
                } else {
                    var data = (this.responseParser || this._defaultResponseParser).call(this, text);
                    this.file.setResultHttpMessage(data instanceof Array ? data[0] : data);
                    this.trigger(_BaseUploader3.default.EVENT_END, [this._xhr.status, data]);
                }
            } else {
                this.file.setResultHttpMessage(text);
                this.trigger(_BaseUploader3.default.EVENT_ERROR, [this._xhr.status, text]);
            }
        }
    }], [{
        key: 'isProgressSupport',
        value: function isProgressSupport() {
            return true;
        }
    }]);

    return XhrUploader;
}(_BaseUploader3.default);

exports.default = XhrUploader;